import Foundation
import UIKit


@objcMembers
@objc(UICollectionViewFlowLinearLayoutImpl)
public class UICollectionViewFlowLinearLayoutImpl: UICollectionViewFlowLayout {
    
    public var transformer: String?
    
    public override func layoutAttributesForElements(in rect: CGRect) -> [UICollectionViewLayoutAttributes]? {
        
        let originalLayoutAttribute = super.layoutAttributesForElements(in:rect)
        guard let collectionView = collectionView else {return originalLayoutAttribute}
        if (transformer == "scale") {
            originalLayoutAttribute?.forEach({ attr in
                attr.frame.size.width = attr.frame.size.width * 0.75
                attr.frame.size.height = attr.frame.size.height * 0.75
                let spacing = minimumLineSpacing
                var distance = CGFloat(0)
                
                if(scrollDirection == .horizontal){
                    distance = abs(
                        collectionView.contentOffset.x +
                        collectionView.contentInset.left +
                        spacing - attr.frame.origin.x
                    )
                }else {
                    distance = abs(
                        collectionView.contentOffset.y +
                        collectionView.contentInset.top +
                        spacing - attr.frame.origin.y
                    )
                }
                
                
                var scale = CGFloat(0)
                
                if(scrollDirection == .horizontal){
                    scale = min(max(1 - distance / collectionView.bounds.size.width, 0.75), 1)
                }else {
                    scale = min(max(1 - distance / collectionView.bounds.size.height, 0.75), 1)
                }
                
                
                attr.transform = CGAffineTransformScale(attr.transform, 1, scale)
            })
        }
        
        return originalLayoutAttribute
        
    }
    
    public override func shouldInvalidateLayout(forBoundsChange newBounds: CGRect) -> Bool {
        return true
    }
    
    
    public override func initialLayoutAttributesForAppearingItem(at itemIndexPath: IndexPath) -> UICollectionViewLayoutAttributes? {
        guard let attrs = super.initialLayoutAttributesForAppearingItem(at: itemIndexPath) else {return nil}
        attrs.alpha = 1
        return attrs
    }
    
    public override func finalLayoutAttributesForDisappearingItem(at itemIndexPath: IndexPath) -> UICollectionViewLayoutAttributes? {
        guard let attrs = super.finalLayoutAttributesForDisappearingItem(at: itemIndexPath) else {return nil}
        attrs.alpha = 1
        return attrs
    }
    
    var onPageChange: ((Int)-> Void)?
    
    
    public override func targetContentOffset(forProposedContentOffset proposedContentOffset: CGPoint, withScrollingVelocity velocity: CGPoint) -> CGPoint {
        var _proposedContentOffset = CGPoint(x: proposedContentOffset.x, y: proposedContentOffset.y)
        var offSetAdjustment: CGFloat = CGFloat.greatestFiniteMagnitude
        
        var horizontalCenter: CGFloat
        
        if(scrollDirection == .horizontal){
            horizontalCenter = CGFloat(proposedContentOffset.x + (self.collectionView!.bounds.size.width / 2.0))
            
        }else{
            horizontalCenter = CGFloat(proposedContentOffset.y + (self.collectionView!.bounds.size.height / 2.0))
            
        }
        
        var targetRect: CGRect
        
        if(scrollDirection == .horizontal){
            targetRect = CGRect(x: proposedContentOffset.x, y: 0.0, width: self.collectionView!.bounds.size.width, height: self.collectionView!.bounds.size.height)
        }else {
            targetRect = CGRect(x: 0.0, y: proposedContentOffset.y, width: self.collectionView!.bounds.size.width, height: self.collectionView!.bounds.size.height)
        }
        
        var array = self.layoutAttributesForElements(in: targetRect)
        
        guard let array = array else {return proposedContentOffset}
        
        
        for layoutAttributes: UICollectionViewLayoutAttributes in array {
            if (layoutAttributes.representedElementCategory == .cell) {
                var itemHorizontalCenter: CGFloat
                
                if(scrollDirection == .horizontal){
                    itemHorizontalCenter = layoutAttributes.center.x
                }else {
                    itemHorizontalCenter = layoutAttributes.center.y
                }
                
                if (abs(itemHorizontalCenter - horizontalCenter) < abs(offSetAdjustment)) {
                    offSetAdjustment = itemHorizontalCenter - horizontalCenter
                }
            }
        }
        
        var nextOffset: CGFloat
        
        if(scrollDirection == .horizontal){
            nextOffset = proposedContentOffset.x + offSetAdjustment
        }else {
            nextOffset = proposedContentOffset.y + offSetAdjustment
        }
        
        repeat {
            if(scrollDirection == .horizontal){
                _proposedContentOffset.x = nextOffset
            }else {
                _proposedContentOffset.y = nextOffset
            }
           
            var deltaX: CGFloat
            var velX: CGFloat
            
            
            if(scrollDirection == .horizontal){
                deltaX = proposedContentOffset.x - self.collectionView!.contentOffset.x
                velX = velocity.x
            }else {
                deltaX = proposedContentOffset.y - self.collectionView!.contentOffset.y
                
                velX = velocity.y
            }
            
        
            
           
            
            if (deltaX == 0.0 || velX == 0 || (velX > 0.0 && deltaX > 0.0) || (velX < 0.0 && deltaX < 0.0)) {
                break
            }
            
            if(scrollDirection == .horizontal){
                if (velocity.x > 0.0) {
                    nextOffset = nextOffset + self.snapStep()
                } else if (velocity.x < 0.0) {
                    nextOffset = nextOffset - self.snapStep()
                }
            }else {
                if (velocity.y > 0.0) {
                    nextOffset = nextOffset + self.snapStep()
                } else if (velocity.y < 0.0) {
                    nextOffset = nextOffset - self.snapStep()
                }
            }
        } while self.isValidOffset(offset: nextOffset)
        
        if(scrollDirection == .horizontal){
            _proposedContentOffset.y = 0.0
        }else {
            _proposedContentOffset.x = 0.0
        }
        
        
        return _proposedContentOffset
    }
    
    
    func isValidOffset(offset: CGFloat) -> Bool {
        return (offset >= CGFloat(self.minContentOffset()) && offset <= CGFloat(self.maxContentOffset()))
    }
    
    func minContentOffset() -> CGFloat {
        if(scrollDirection == .vertical){
            return -CGFloat(self.collectionView!.contentInset.top)
        }
        return -CGFloat(self.collectionView!.contentInset.left)
    }
    
    func maxContentOffset() -> CGFloat {
        if(scrollDirection == .vertical){
            return CGFloat(self.minContentOffset() + self.collectionView!.contentSize.height - self.itemSize.height)
        }
        return CGFloat(self.minContentOffset() + self.collectionView!.contentSize.width - self.itemSize.width)
    }
    
    func snapStep() -> CGFloat {
        if(scrollDirection == .vertical){
            return self.itemSize.height + self.minimumLineSpacing
        }
        return self.itemSize.width + self.minimumLineSpacing
    }
}



@objcMembers
@objc(PagerUtils)
public class PagerUtils: NSObject {
    
    
    public static func updateCollection(_ update: [String: Any],_ collection : UICollectionView) {
        DispatchQueue.main.async {
            collection.performBatchUpdates({
                var changes: [IndexPath] = []
                var addedCount: Int = 0
                var removedCount: Int = 0
                let index = update["index"] as! Int
                switch update["action"] as! String {
                case "add":
                    addedCount = update["addedCount"] as! Int
                    for i in 0...addedCount - 1 {
                        changes.append(
                            IndexPath(row: index + i, section: 0)
                        )
                    }
                    collection.insertItems(at:changes)
                case "delete":
                    removedCount = update["removedCount"] as! Int
                    for i in 0...removedCount - 1 {
                        changes.append(
                            IndexPath(row: index + i, section: 0)
                        )
                    }
                    collection.deleteItems(at: changes)
                case "update":
                    collection.reloadItems(at: [
                        IndexPath(row: index, section: 0)
                    ])
                    
                case "splice":
                    removedCount = update["removedCount"] as! Int
                    addedCount = update["addedCount"] as! Int
                    if (removedCount > 0) {
                        for i in 0...removedCount - 1 {
                            changes.append(
                                IndexPath(row: index + i, section: 0)
                            )
                        }
                        collection.deleteItems(at: changes)
                    } else {
                        let addedArray: [IndexPath] = []
                        for i in 0...addedCount - 1 {
                            changes.append(
                                IndexPath(row: index + i, section: 0)
                            )
                        }
                        collection.insertItems(at:addedArray)
                    }
                    
                    // case "change":
                default:
                    // NO-OP
                    changes = []
                }
            }, completion: nil)
        }
    }
}
